/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.connectors.jcr.adapters.local;

import java.net.MalformedURLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

import org.apache.commons.logging.Log;
import org.exoplatform.container.StandaloneContainer;
import org.exoplatform.services.jcr.impl.jndi.BindableRepositoryImpl;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id: JcrResourceAdapter.java 5651 2006-05-22 13:53:09Z geaz $
 */

public class JcrResourceAdapter implements ResourceAdapter {
  
  private static Log log = ExoLogger.getLogger("jcr.JcrResourceAdapter");
  
  String repositoryConfig;

  String containerConfig;

  String bindName;

  /* (non-Javadoc)
   * @see javax.resource.spi.ResourceAdapter#start(javax.resource.spi.BootstrapContext)
   */
  public synchronized void start(BootstrapContext ctx)
      throws ResourceAdapterInternalException {
    
    log.info("repositoryConfig: " + repositoryConfig);
    log.info("containerConfig: " + containerConfig);
    log.info("bindName: " + bindName);
    Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
    try {
      StandaloneContainer.setConfigurationURL(containerConfig);
    } catch (MalformedURLException e) {
      log.warn("Invalid containerConfig URL, ignored: "+containerConfig);
    }
    
    try {
      StandaloneContainer sc = StandaloneContainer.getInstance();
      InitialContext jndiContext = new InitialContext();
      jndiContext.bind(bindName, new BindableRepositoryImpl(sc, repositoryConfig));
    } catch (Exception e) {
      log.error("BINDING ERROR: " + e);
      e.printStackTrace();
    }
  }

  /* (non-Javadoc)
   * @see javax.resource.spi.ResourceAdapter#stop()
   */
  public void stop() {
    try {
      InitialContext jndiContext = new InitialContext();
      jndiContext.unbind(bindName);
    } catch (Exception e) {
    }
  }

  /* (non-Javadoc)
   * @see javax.resource.spi.ResourceAdapter#getXAResources(javax.resource.spi.ActivationSpec[])
   */
  public XAResource[] getXAResources(ActivationSpec[] specs)
      throws ResourceException {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.resource.spi.ResourceAdapter#endpointActivation(javax.resource.spi.endpoint.MessageEndpointFactory, javax.resource.spi.ActivationSpec)
   */
  public void endpointActivation(MessageEndpointFactory endpointFactory,
      ActivationSpec spec) throws ResourceException {
  }

  /* (non-Javadoc)
   * @see javax.resource.spi.ResourceAdapter#endpointDeactivation(javax.resource.spi.endpoint.MessageEndpointFactory, javax.resource.spi.ActivationSpec)
   */
  public void endpointDeactivation(MessageEndpointFactory endpointFactory,
      ActivationSpec spec) {
  }
  
  public void setRepositoryConfig(String prop) {
    this.repositoryConfig = prop;
  }

  public void setContainerConfig(String prop) {
    this.containerConfig = prop;
  }

  public void setBindName(String prop) {
    this.bindName = prop;
  }

  void bind(String name, Object obj) {
    try {
      Context ctx = new InitialContext();
      ctx.rebind(name, obj);
    } catch (Exception e) {
    }
  }


}
