/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.jcr.web;

import java.io.InputStream;
import java.net.MalformedURLException;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.exoplatform.container.StandaloneContainer;

/**
 * Created by The eXo Platform SARL        .<br/>
 * Servlet context initializer that initializes standalone container at the context startup time.
 * To activate this your web.xml have to be configured like:
 * <listener>
 * <listener-class>org.exoplatform.frameworks.web.common.StandaloneContainerInitializedListener</listener-class>
 * </listener>
 * You may also specify an URL to the configuration.xml stored the configuration for StandaloneContainer
 * as servlet's init parameter called 'org.exoplatform.container.standalone.config'
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: StandaloneContainerInitializedListener.java 5800 2006-05-28 18:03:31Z geaz $
 */

public class StandaloneContainerInitializedListener implements
    ServletContextListener {
  
  /**
   * org.exoplatform.container.standalone.config
   */
  private static final String CONF_URL_PARAMETER = "org.exoplatform.container.standalone.config";
  
  
  /* (non-Javadoc)
   * @see javax.servlet.ServletContextListener#contextInitialized(javax.servlet.ServletContextEvent)
   */
  public void contextInitialized(ServletContextEvent event) {
    
    String configurationURL = event.getServletContext().
    getInitParameter(CONF_URL_PARAMETER);
    if(configurationURL == null)
      configurationURL = "conf/exo-configuration.xml";
    try {
      StandaloneContainer.setConfigurationURL(configurationURL);
    } catch (MalformedURLException e1) {
    }
 
    //System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>> "+configurationURL);
    try {
      StandaloneContainer container = StandaloneContainer.
      getInstance(Thread.currentThread().getContextClassLoader());
      event.getServletContext().setAttribute(WebConstants.EXO_CONTAINER, 
          container);
      //System.out.println("CONTAINER ------"+      Thread.currentThread().getContextClassLoader());
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletContextListener#contextDestroyed(javax.servlet.ServletContextEvent)
   */
  public void contextDestroyed(ServletContextEvent event) {
  }
}
