/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core.nodetype;

import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: WorkspaceNTImpl.java 5150 2006-04-17 06:44:21Z geaz $
 */

public class WorkspaceNTImpl implements NodeType {
  
  private NodeTypeImpl nodeType;
  private SessionImpl session;

  public WorkspaceNTImpl(NodeTypeImpl nodeType, SessionImpl session) {
    super();
    this.nodeType = nodeType;
    this.session = session;
  }

  public String getName() {
    
    String n = null;
    try {
      n = session.getLocationFactory().createJCRName(getQName()).getAsString();
    } catch (RepositoryException e) {
      // should never happen
      throw new RuntimeException("TYPE NAME >>> "+n+" "+e, e);
    }
    return n;
  }

  public NodeType[] getDeclaredSupertypes() {
    return nodeType.getDeclaredSupertypes();
  }

  public NodeType[] getSupertypes() {
    return nodeType.getSupertypes();
  }

  public boolean canAddChildNode(String childNodeName, String nodeTypeName) {
    return nodeType.canAddChildNode(childNodeName, nodeTypeName);
  }

  public boolean canAddChildNode(String childNodeName) {
    return nodeType.canAddChildNode(childNodeName);
  }

  public boolean canRemoveItem(String itemName) {
    return nodeType.canRemoveItem(itemName);
  }

  public boolean canSetProperty(String propertyName, Value value) {
    return nodeType.canSetProperty(propertyName, value);
  }

  public boolean canSetProperty(String propertyName, Value[] values) {
    return nodeType.canSetProperty(propertyName, values);
  }

  public NodeDefinition[] getChildNodeDefinitions() {
    return nodeType.getChildNodeDefinitions();
  }

  public NodeDefinition[] getDeclaredChildNodeDefinitions() {
    return nodeType.getDeclaredChildNodeDefinitions();
  }

  public PropertyDefinition[] getDeclaredPropertyDefinitions() {
    return nodeType.getDeclaredPropertyDefinitions();
  }

  public String getPrimaryItemName() {
    return nodeType.getPrimaryItemName();
  }

  public PropertyDefinition[] getPropertyDefinitions() {
    return nodeType.getPropertyDefinitions();
  }


  public boolean hasOrderableChildNodes() {
    return nodeType.hasOrderableChildNodes();
  }

  public boolean isMixin() {
    return nodeType.isMixin();
  }

  public boolean isNodeType(String nodeTypeName) {
    return nodeType.isNodeType(nodeTypeName);
  }
  
  public boolean equals(Object obj) {
    return nodeType.equals(obj);
  }

  InternalQName getQName() {
    return nodeType.getQName();
  }

}
