/**
 **************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.jndi;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import javax.jcr.Credentials;
import javax.jcr.LoginException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.NoSuchWorkspaceException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.StringRefAddr;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeTypeManager;
import org.exoplatform.services.jcr.impl.RepositoryServiceImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id: BindableRepositoryImpl.java 5484 2006-05-11 07:09:32Z geaz $
 */

public class BindableRepositoryImpl implements Serializable, Referenceable, ManageableRepository {


  private static transient ExoContainer topContainer = null;
  private transient RepositoryImpl delegatee = null;
  private String configName;

  public BindableRepositoryImpl(ExoContainer container, String config)
      throws RepositoryException, RepositoryConfigurationException, FileNotFoundException, Exception {
    topContainer = container;
    configName = config;
    init();
  }

  private void init() throws RepositoryException, RepositoryConfigurationException, RepositoryException,
    FileNotFoundException, Exception {
    //RepositoryEntry config = null;
    RepositoryServiceImpl rs = (RepositoryServiceImpl)topContainer.getComponentInstanceOfType(RepositoryServiceImpl.class);
    delegatee = (RepositoryImpl)rs.getRepository(configName);
  }
  
  public String[] getDescriptorKeys() {
    return delegatee.getDescriptorKeys();
  }

  public String getDescriptor(String key) {
    return delegatee.getDescriptor(key);
  }

  public Session login(Credentials credentials) throws LoginException,
      NoSuchWorkspaceException, RepositoryException {
    return delegatee.login(credentials);
  }

  public Session login(String workspaceName) throws LoginException,
      NoSuchWorkspaceException, RepositoryException {
    return delegatee.login(workspaceName);
  }

  public Session login() throws LoginException, NoSuchWorkspaceException,
      RepositoryException {
    return delegatee.login();
  }

  public Session login(Credentials credentials, String workspaceName)
      throws LoginException, NoSuchWorkspaceException, RepositoryException {
    return delegatee.login(credentials, workspaceName);
  }
  
  public Session getSystemSession(String workspaceName)
      throws RepositoryException {
    return delegatee.getSystemSession(workspaceName);
  }

  public String[] getWorkspaceNames() {
    return delegatee.getWorkspaceNames();
  }

  public ExtendedNodeTypeManager getNodeTypeManager() {
    return delegatee.getNodeTypeManager();
  }
  
  public NamespaceRegistry getNamespaceRegistry() {
    return delegatee.getNamespaceRegistry();
  }

  public void initWorkspace(String workspaceName, String rootNodeType)
      throws RepositoryException {
    delegatee.initWorkspace(workspaceName, rootNodeType);
  }

  public boolean isWorkspaceInitialized(String workspaceName) {
    return delegatee.isWorkspaceInitialized(workspaceName);
  }

  public RepositoryEntry getConfiguration() {
    return delegatee.getConfiguration();
  }

  public String getSystemWorkspaceName() {
    return delegatee.getSystemWorkspaceName();
  }

  public String getName() {
    return delegatee.getName();
  }

  public SessionImpl getSystemSession() throws RepositoryException {
    return delegatee.getSystemSession();
  }

  private void writeObject(ObjectOutputStream out) throws IOException {
    // delegate to default implementation
    out.defaultWriteObject();
  }

  private void readObject(ObjectInputStream in)
    throws IOException, ClassNotFoundException, RepositoryException, RepositoryConfigurationException, Exception {
    // delegate deserialization to default implementation
    in.defaultReadObject();
    // initialize reconstructed instance
    init();
  }

  public Reference getReference() throws NamingException {
    Reference ref = new Reference(
      BindableRepositoryImpl.class.getName(),
      BindableRepositoryFactory.class.getName(),
      null);
    ref.add(new StringRefAddr(BindableRepositoryFactory.REPOSITORYNAME_ADDRTYPE, configName));
    return ref;
  }
}
