/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.sql.Connection;
import java.sql.SQLException;

import javax.jcr.RepositoryException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.exoplatform.services.database.jdbc.DBSchemaCreator;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.jcr.storage.WorkspaceStorageConnection;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: JDBCWorkspaceDataContainer.java 5800 2006-05-28 18:03:31Z geaz $
 */
public class JDBCWorkspaceDataContainer extends
    WorkspaceDataContainerBase {

  protected static Log log = ExoLogger.getLogger("jcr.JDBCWorkspaceDataContainer");
  
  protected String containerName;
  protected String sourceName;
  protected boolean multiDB;
  //protected GlobalContextProvider contextProvider;
  //private DBSchemaCreator dbCreator;
  private DataSource ds;
  
  public JDBCWorkspaceDataContainer(WorkspaceEntry wsConfig, DBSchemaCreator dbCreator) 
  throws RepositoryConfigurationException, NamingException {
    
    this.containerName = wsConfig.getName();
    this.sourceName = wsConfig.getContainer().getParameterValue("sourceName");
    //this.contextProvider = contextProvider;
    
    multiDB = Boolean.parseBoolean(
        wsConfig.getContainer().getParameterValue("multi-db"));
    
    //Context context = contextProvider.getContext();
    Context context = new InitialContext();
    ds = (DataSource) context.lookup(sourceName);

    
    //this.dbCreator = dbCreator;
    log.info("JDBCWorkspaceDataContainer (cEntry): " + containerName + ", dataSource: " + sourceName);
  }

  
  public WorkspaceStorageConnection openConnection() throws RepositoryException {
    
    try {

      Connection conn = ds.getConnection();
      
      conn.setAutoCommit(false);
      
      if (multiDB) 
        return new MultiDbJDBCConnection(conn, containerName);
      else 
        return new SingleDbJDBCConnection(conn, containerName);

    } catch (SQLException e) {
      throw new RepositoryException(e);
    }
  }

  /**
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getName()
   */
  public String getName() {
    return containerName;
  }

  public String getInfo() {
    String str = "Info: JDBC based JCR Workspace Data container \n";
    //str += "\tName: " + containerName + "\n";
    //str += "\tHibernate configuration: \n";
    //str += "\tDatasource name (hibernate.datasource.name): ???"
    //    + "\n";
    return str;
  }

  public boolean isMultiDB() {
    return multiDB; 
  }
  

}