/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.storage.rdb.container;

import java.util.List;

import javax.jcr.InvalidItemStateException;
import javax.jcr.ItemExistsException;
import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.NodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.PropertyRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBItemRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBNodeRecord;
import org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBPropertyRecord;
import org.hibernate.NonUniqueObjectException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

/**
 * Created by The eXo Platform SARL
 * Author : Peter Nedonosko
 *          peter.nedonosko@exoplatform.com.ua
 * 20.03.2006
 * @version $Id: HibernateSingleStorageConnection.java 5330 2006-04-28 16:00:52Z peterit $
 */

 // Warning FIND_PROPS_BY_VALUE is not correct for single container,
 // mind containerName 
public class HibernateSingleStorageConnection extends HibernateStorageConnection {
  
  protected static String FIND_ITEM_BY_PATH = "from SingleRDBItemRecord as item where item.path=? and item.containerName=?";

  protected static String FIND_NODE_BY_PATH = "from SingleRDBNodeRecord as item where item.path=? and item.containerName=?";

  protected static String FIND_PROPERTY_BY_PATH = "from SingleRDBPropertyRecord as item where item.path=? and item.containerName=?";

  protected static String FIND_PROPS_BY_TYPE_AND_VALUE = "select distinct prop from SingleRDBValueRecord as val, SingleRDBPropertyRecord as prop inner join val.property where prop.valuesType=? and val.data=? and prop.containerName=?";
  
  protected static String FIND_PROPS_BY_VALUE = "from SingleRDBValueRecord as val where val.data=?";
         
  protected static String FIND_NODES_BY_PARENTUUID = "from SingleRDBNodeRecord as item where item.parentNode=? and item.containerName=?";
  
  protected static String FIND_PROPERTIES_BY_PARENTUUID = "from SingleRDBPropertyRecord as item where item.parentNode=? and item.containerName=?";

  protected static String FIND_NODESCOUNT_BY_PARENTUUID = "select count(*) from SingleRDBNodeRecord as item where item.parentNode=? and item.containerName=?";
  
  protected static String FIND_PROPERTIESCOUNT_BY_PARENTUUID = "select count(*) from SingleRDBPropertyRecord as item where item.parentNode=? and item.containerName=?";
  
  private String containerName;
  
  public HibernateSingleStorageConnection(String containerName, String sourceName, 
      SessionFactory sessionFactory) {
   
    super(sourceName, sessionFactory);
    this.containerName = containerName;
    log.debug("HibernateMultyStorageConnection (cEntry) " + containerName
        + " hibernate container = " + sourceName);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#addNodeRecord(org.exoplatform.services.jcr.datamodel.NodeData, org.hibernate.Session)
   */
  synchronized protected NodeRecord addNodeRecord(NodeData data,
      Session hibernateSession) throws RepositoryException,
      ItemExistsException, InvalidItemStateException {

    NodeRecord parent = findParentNodeRecord(data);
    SingleRDBNodeRecord nodeRecord = new SingleRDBNodeRecord();
    nodeRecord.setId(id2cid(data.getUUID()));
    nodeRecord.setPath(data.getQPath().getAsString());
    nodeRecord.setParentNode(parent);
    nodeRecord.setOrderNumber(data.getOrderNumber());
    
    nodeRecord.setContainerName(this.containerName);
    
    if (parent != null) // not root
      parent.addChildNode(nodeRecord);

    try {
      hibernateSession.persist(nodeRecord);
    } catch (NonUniqueObjectException e) {
      String sameUuidPath = "";
      try {
        NodeRecord sameUuid = findNodeByUUID(data.getUUID(), hibernateSession);
        if (sameUuid != null) { 
          sameUuidPath = sameUuid.getPath();
        }
      } catch(Exception e1) {
        log.info("addNodeRecord(): same uuid node search ERROR: " + data.getUUID(), e); 
      }
      throw new ItemExistsException("Item with UUID "+data.getUUID() + "  " + sameUuidPath + "  already exists");
    } finally {
      //toLog("  addNodeRecord(NodeData) fin: " + hibernateSession);
    }

    return nodeRecord;

  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#addPropertyRecord(org.exoplatform.services.jcr.datamodel.PropertyData, org.exoplatform.services.jcr.impl.storage.rdb.container.data.NodeRecord, org.hibernate.Session)
   */
  synchronized protected PropertyRecord addPropertyRecord(PropertyData prop,
      NodeRecord parent, Session hibernateSession)
      throws RepositoryException, ItemExistsException {

    SingleRDBPropertyRecord propertyRecord = new SingleRDBPropertyRecord();
    propertyRecord.setId(id2cid(prop.getUUID()));
    propertyRecord.setPath(prop.getQPath().getAsString());
    propertyRecord.setType(prop.getType());
    propertyRecord.setValues(prop.getValues());
    propertyRecord.setMultiValued(prop.isMultiValued());
    
    propertyRecord.setContainerName(this.containerName);


    parent.addProperty(propertyRecord);

    hibernateSession.persist(propertyRecord);
    
    log.debug("RDB Container added: " + propertyRecord);

    return propertyRecord;

  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findItemByPath(java.lang.String, org.hibernate.Session)
   */
  protected ItemRecord findItemByPath(String path, Session hibernateSession) {
    
    SingleRDBItemRecord r = (SingleRDBItemRecord) hibernateSession.createQuery(FIND_ITEM_BY_PATH)
      .setString(0, path)
      .setString(1, containerName)
      .uniqueResult();
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findItemByUUID(java.lang.String, org.hibernate.Session)
   */
  protected ItemRecord findItemByUUID(String uuid, Session hibernateSession) {
    
    SingleRDBItemRecord r = (SingleRDBItemRecord) hibernateSession.get(SingleRDBItemRecord.class,
        id2cid(uuid));
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findNodeByPath(java.lang.String, org.hibernate.Session)
   */
  protected NodeRecord findNodeByPath(String path, Session hibernateSession) {
    
    Query query = hibernateSession.createQuery(FIND_NODE_BY_PATH);
    query.setString(0, path);
    query.setString(1, containerName);
    SingleRDBNodeRecord r = (SingleRDBNodeRecord) query.uniqueResult();
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findNodeByUUID(java.lang.String, org.hibernate.Session)
   */
  protected NodeRecord findNodeByUUID(String uuid, Session hibernateSession) {
    
    SingleRDBNodeRecord r = (SingleRDBNodeRecord) hibernateSession.get(SingleRDBNodeRecord.class,
        id2cid(uuid));
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findPropertyByPath(java.lang.String, org.hibernate.Session)
   */
  protected PropertyRecord findPropertyByPath(String path,
      Session hibernateSession) {
    
    SingleRDBPropertyRecord r = (SingleRDBPropertyRecord) hibernateSession.createQuery(FIND_PROPERTY_BY_PATH)
    	.setString(0, path)
	.setString(1, containerName)
	.uniqueResult();
    
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findPropertyByUUID(java.lang.String, org.hibernate.Session)
   */
  protected PropertyRecord findPropertyByUUID(String uuid,
      Session hibernateSession) {
    
    SingleRDBPropertyRecord r = (SingleRDBPropertyRecord) hibernateSession.get(
        SingleRDBPropertyRecord.class, id2cid(uuid));
    return r;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#findReferences(java.lang.String, org.hibernate.Session)
   */
  protected List findReferences(String uuid, Session hibernateSession) {
    // [PN] try use first commented block (FIND_PROPS_BY_TYPE_AND_VALUE) 
    // instead current (FIND_PROPS_BY_VALUE)
    
    //  return hibernateSession.createQuery(FIND_PROPS_BY_TYPE_AND_VALUE)
    //    .setInteger(0, PropertyType.REFERENCE).setBinary(1, uuid.getBytes())
    //    .list();
    
    List references = hibernateSession.createQuery(FIND_PROPS_BY_VALUE).setBinary(0,
        uuid.getBytes()).list();
    /* // [PN] 23.03.06 Prev impl.
    List props = new ArrayList();
    for (int i = 0; i < references.size(); i++) {
      ValueRecord rec = (ValueRecord) references.get(i);
      if (rec.getProperty().getType() == PropertyType.REFERENCE) {
        props.add(rec.getProperty());
      }
    }
     */
    return references;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#getChildNodesByParentUUID(java.lang.String, org.hibernate.Session)
   */
  protected List getChildNodesByParentUUID(String parentUUID, Session hibernateSession) {
    List childNodes = hibernateSession.createQuery(FIND_NODES_BY_PARENTUUID)
      .setString(0, id2cid(parentUUID))
      .setString(1, containerName)
      .list();
    return childNodes;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#getChildPropertiesByParentUUID(java.lang.String, org.hibernate.Session)
   */
  protected List getChildPropertiesByParentUUID(String parentUUID, Session hibernateSession) {
    List childProperties = hibernateSession.createQuery(FIND_PROPERTIES_BY_PARENTUUID)
      .setString(0, id2cid(parentUUID))
      .setString(1, containerName)
      .list();
    return childProperties;
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateWorkspaceDataContainer#getChildNodesCountByParentUUID(java.lang.String, org.hibernate.Session)
   */
  @Override
  protected int getChildNodesCountByParentUUID(String parentUUID, Session hibernateSession) {
    Integer r = (Integer) hibernateSession.createQuery(FIND_NODESCOUNT_BY_PARENTUUID)
      .setString(0, id2cid(parentUUID))
      .setString(1, containerName)
      .uniqueResult();
    return r.intValue();
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.HibernateStorageConnection#getChildPropertiesCountByParentUUID(java.lang.String, org.hibernate.Session)
   */
  @Override
  protected int getChildPropertiesCountByParentUUID(String parentUUID, Session hibernateSession) {
    Integer r = (Integer) hibernateSession.createQuery(FIND_PROPERTIESCOUNT_BY_PARENTUUID)
      .setString(0, id2cid(parentUUID))
      .setString(1, containerName)
      .uniqueResult();
    return r.intValue();
  }  
  
  private String id2cid(String uuid) {
    return containerName+uuid;
  }

  // experimental
  protected void initIndexes() {
    // JCR_SITEM table
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SITEM_ID ON JCR_SITEM(ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SITEM_ID index " + e.getMessage());
//    }
   
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SITEM_PATH ON JCR_SITEM(PATH)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SITEM_PATH index " + e.getMessage());
//    }
    
//    // JCR_SNODE table
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SNODE_ID ON JCR_SNODE(ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SNODE_ID index " + e.getMessage());
//    }
//    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SNODE_PID ON JCR_SNODE(PARENT_ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SNODE_PID index " + e.getMessage());
//    }
//    
//    // JCR_SPROPERTY table
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SPROPERTY_ID ON JCR_SPROPERTY(ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SPROPERTY_ID index " + e.getMessage());
//    }
//    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SPROPERTY_PID ON JCR_SPROPERTY(PARENT_ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SPROPERTY_PID index " + e.getMessage());
//    }
//    
//    
//    // JCR_SVALUE table
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SVALUE_ID ON JCR_SVALUE(ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SVALUE_PID index " + e.getMessage());
//    }
//    
//    try {
//      hibernateSession.connection().createStatement().executeUpdate("CREATE INDEX JCR_IDX_SVALUE_PID ON JCR_SVALUE(PROPERTY_ID)");
//    } catch(Exception e) {
//      log.info("Error init JCR_IDX_SVALUE_PID index " + e.getMessage());
//    }
  }  
}
