/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.rdb.container;

import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.container.component.ComponentLifecycle;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.storage.WorkspaceDataContainerBase;
import org.exoplatform.services.jcr.storage.WorkspaceStorageConnection;
import org.hibernate.SessionFactory;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: HibernateWorkspaceDataContainer.java 5307 2006-04-27 13:55:57Z peterit $
 */
public class HibernateWorkspaceDataContainer extends
    WorkspaceDataContainerBase implements 
    ComponentLifecycle {

  protected static final Log log = LogFactory.getLog("jcr.HibernateWorkspaceDataContainer");

  protected String containerName;
  protected String sourceName;
  protected boolean multiplicityAllowed;
  protected HibernateService hibernateService = null;
  protected ExoContainerContext containerContext;
  
  public HibernateWorkspaceDataContainer(WorkspaceEntry wsConfig, 
      ExoContainerContext containerContext) 
  throws RepositoryConfigurationException {
    
    this.containerName = wsConfig.getName();
    sourceName = wsConfig.getContainer().getParameterValue("sourceName");
    
    String multiplicityAllowedStr = wsConfig.getContainer().getParameterValue("multiplicity");
    multiplicityAllowed = Boolean.parseBoolean(multiplicityAllowedStr);
    
    this.containerContext = containerContext;
    
    log.info("HibernateWorkspaceDataContainer (cEntry)" + containerName
        + " hibernate container =" + sourceName);
  }

  /** (non-Javadoc)
   * @see org.picocontainer.Startable#start()
  */
  /*
  public void start() {
    ExoContainer container = this.containerContext.getContainer();
    if (sourceName == null || sourceName.length() == 0 || "default".equals(sourceName)) {
      hibernateService = (HibernateService) container
          .getComponentInstanceOfType(HibernateService.class);
    } else {
      hibernateService = (HibernateService) container
          .getComponentInstance(sourceName);
    }
    if (hibernateService == null) {
       log.fatal("Cant obtain hibernate service for "
            + sourceName);
       throw new RuntimeException("Cant obtain hibernate service for "
           + sourceName);
    }
    log.info("HibernateWorkspaceDataContainer is started using hibernate container =" + sourceName);
  }

  public void stop() {
  }
  */
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getConnection()
   */
  public WorkspaceStorageConnection openConnection() {
    
    WorkspaceStorageConnection wsCon;
    
    if (multiplicityAllowed) {
      wsCon = new HibernateMultiStorageConnection( 
          sourceName, hibernateService.getSessionFactory());
    } else {
      wsCon = new HibernateSingleStorageConnection(containerName, 
          sourceName, hibernateService.getSessionFactory());
    }
    
    return wsCon;
  }

  protected void initRDBIndexes() {
    try {
      WorkspaceStorageConnection con = openConnection();
      try {
        if (con instanceof HibernateStorageConnection) {
          ((HibernateStorageConnection)con).initIndexes();
        }  
      } finally {
        con.rollback();
      }
    } catch(RepositoryException e) {
      log.error("initRDBIndexes() error", e);
    }
  }
  
  /**
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.storage.WorkspaceDataContainer#getName()
   */
  public String getName() {
    return containerName;
  }

  public String getInfo() {
    String str = "Info: Relational Database (Hibernate 3) based container ver1.0 \n";
    str += "\tName: " + containerName + "\n";
    str += "\tHibernate configuration: \n";
    str += "\tDatasource name (hibernate.datasource.name): ???"
    //				+ hibernateConfiguration
        //						.getHibernateProperty("hibernate.datasource.name")
        + "\n";
    return str;
  }

  public boolean isMultiplicityAllowed() {
    return multiplicityAllowed; 
  }

  protected SessionFactory getSessionFactory() {
    return hibernateService.getSessionFactory();
  }

  
  ////// ComponentLifecycle
    public void initComponent(ExoContainer container) throws Exception {

      if (sourceName == null || sourceName.length() == 0 || "default".equals(sourceName)) {
        hibernateService = (HibernateService) container
            .getComponentInstanceOfType(HibernateService.class);
      } else {
        hibernateService = (HibernateService) container
            .getComponentInstance(sourceName);
      }

      if (hibernateService == null)
        throw new RepositoryException("Cant obtain hibernate service for "
            + sourceName);
      
      // [PN] 21.04.06
      initRDBIndexes();
    }
    
  
    public void startComponent(ExoContainer arg0) throws Exception {
    }

    public void stopComponent(ExoContainer arg0) throws Exception {
    }

    public void destroyComponent(ExoContainer arg0) throws Exception {
    }
    
    

}
