package org.exoplatform.services.jcr;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFactory;
import javax.jcr.Workspace;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.exoplatform.container.StandaloneContainer;
import org.exoplatform.services.jcr.impl.core.CredentialsImpl;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: BaseStandaloneTest.java 5456 2006-05-10 08:28:33Z peterit $
 */
public abstract class BaseStandaloneTest extends TestCase {
  protected Log log;

  protected static String TEMP_PATH = "./temp/fsroot";

  protected static String WORKSPACE = "ws";

  protected SessionImpl session;

  protected RepositoryImpl repository;

  protected CredentialsImpl credentials;

  protected Workspace workspace;

  protected RepositoryService repositoryService;

  protected LogService logService;

  protected Node root;

  protected ValueFactory valueFactory;
  
  protected StandaloneContainer container;

  public void setUp() throws Exception {
    
    StandaloneContainer.setConfigurationPath("src/java/conf/standalone/test-configuration.xml");

    //System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
  	
    container = StandaloneContainer.getInstance();
  	
    //logService = (LogService)container.getComponentInstanceOfType(LogService.class);
    //log = logService.getLog("org.exoplatform.services.jcr");
    //logService.setLogLevel("org.exoplatform.services.jcr", LogService.DEBUG, true);
    
    log = ExoLogger.getLogger("jcr.JCRTest");

    if (System.getProperty("java.security.auth.login.config") == null)
      System.setProperty("java.security.auth.login.config",
          "src/resources/login.conf");
	
    credentials = new CredentialsImpl("exo", "exo".toCharArray());

    repositoryService = (RepositoryService) container
        .getComponentInstanceOfType(RepositoryService.class);

    repository = (RepositoryImpl) repositoryService.getRepository();

    //repository.getContainer().start();

    if (!((RepositoryImpl) repository).isWorkspaceInitialized("ws"))
      ((RepositoryImpl) repository).initWorkspace("ws", "nt:unstructured");
//    
//    if (!((RepositoryImpl) repository).isWorkspaceInitialized("ws1"))
//      ((RepositoryImpl) repository).initWorkspace("ws1", "nt:unstructured");
//
//    if (!((RepositoryImpl) repository).isWorkspaceInitialized("ws2"))
//      ((RepositoryImpl) repository).initWorkspace("ws2", "nt:unstructured");

    session = (SessionImpl) repository.login(credentials, "ws");
    workspace = session.getWorkspace();
    root = session.getRootNode();
    valueFactory = session.getValueFactory();

    //logService.setLogLevel("org.exoplatform.services.jcr", LogService.DEBUG, true);

    initRepository();
  }

  protected void tearDown() throws Exception {

    // [PN] 05.05.06
    
    log.info("tearDown() BEGIN " + getClass().getName() + "." + getName());

    if (session != null) {
      try {
        session.refresh(false);
        if (root.hasNodes()) {
          // clean test root
          for (NodeIterator children = root.getNodes(); children.hasNext();) {
            Node node = children.nextNode();
            if (!node.getPath().startsWith("/jcr:system")) {
              node.remove();
            }
          }
          session.save();
        }
      } catch (Exception e) {
        e.printStackTrace();
        log.error("===== Exception in tearDown() " + e.toString());
      } finally {
        session.logout();
      }
    }
    super.tearDown();

    //log.info("tearDown() END " + getClass().getName() + "." + getName());
  }  
  
  protected abstract String getRepositoryName();

  public void initRepository() throws RepositoryException {
  }

}
